import * as tslib_1 from "tslib";
import "../../FabricInit";
import * as React from 'react';
import { BaseComponent } from 'office-ui-fabric-react/lib/Utilities';
import { ObservableLike } from '../../Core/Observable';
/**
 * Handles subscription to properties that are IObservableValues, so that components don't have to handle on their own.
 *
 * Usage:
 *
 * <ObservableComponent myObservableValue={observableValue} className='foo'>
 *   <MyComponent myObservableValue='' />
 * </ObservableComponent>
 *
 * Your component will get re-rendered with the new value of myObservableValue whenever that value changes.
 * Additionally, any additional props set on ObservableComponent will also get passed down.
 */
var ObservableComponent = /** @class */ (function (_super) {
    tslib_1.__extends(ObservableComponent, _super);
    function ObservableComponent(props) {
        var _this = _super.call(this, props) || this;
        _this._observableSubscriptionDelegates = {};
        var state = {};
        for (var propName in _this.props) {
            state[propName] = ObservableLike.getValue(_this.props[propName]);
        }
        _this.state = state;
        return _this;
    }
    ObservableComponent.prototype.componentWillMount = function () {
        for (var propName in this.props) {
            this._subscribe(propName, this.props);
        }
    };
    ObservableComponent.prototype.componentWillUnmount = function () {
        for (var propName in this.props) {
            this._unsubscribe(propName, this.props);
        }
    };
    ObservableComponent.prototype.componentWillReceiveProps = function (nextProps) {
        var oldProps = this.props;
        var newState = tslib_1.__assign({}, this.state);
        var stateChanged = false;
        for (var propName in oldProps) {
            var oldValue = oldProps[propName];
            var newValue = nextProps[propName];
            if (oldValue !== newValue) {
                this._unsubscribe(propName, this.props);
                this._subscribe(propName, nextProps);
                newState[propName] = ObservableLike.getValue(newValue);
                stateChanged = true;
            }
        }
        for (var propName in nextProps) {
            var oldValue = oldProps[propName];
            if (oldValue === undefined) {
                var newValue = nextProps[propName];
                this._subscribe(propName, nextProps);
                newState[propName] = ObservableLike.getValue(newValue);
                stateChanged = true;
            }
        }
        if (stateChanged) {
            this.setState(newState);
        }
    };
    ObservableComponent.prototype.render = function () {
        var _this = this;
        if (this.props.children && React.Children.count(this.props.children) <= 1) {
            var childrenWithProps = React.Children.map(this.props.children, function (child) {
                if (child === null) {
                    return null;
                }
                var childElement = child;
                var newProps = {};
                for (var key in _this.state) {
                    if (key !== 'children') {
                        newProps[key] = _this.state[key];
                    }
                }
                newProps['children'] = childElement.props.children;
                var childWithProps = React.cloneElement(childElement, newProps);
                return childWithProps;
            });
            if (childrenWithProps.length === 0) {
                return null;
            }
            return childrenWithProps[0];
        }
        else {
            throw new Error("More than one child component cannot be passed to ObservableComponent");
        }
    };
    ObservableComponent.prototype._subscribe = function (propName, props) {
        var observableValue = props[propName];
        if (ObservableLike.isObservable(observableValue)) {
            var delegate = this._onValueChanged.bind(this, propName);
            ObservableLike.subscribe(observableValue, delegate);
            this._observableSubscriptionDelegates[propName] = delegate;
        }
    };
    ObservableComponent.prototype._unsubscribe = function (propName, props) {
        var observableValue = this.props[propName];
        if (ObservableLike.isObservable(observableValue)) {
            var delegate = this._observableSubscriptionDelegates[propName];
            ObservableLike.unsubscribe(observableValue, delegate);
            delete this._observableSubscriptionDelegates[propName];
        }
    };
    ObservableComponent.prototype._onValueChanged = function (propName, value, action) {
        var _a;
        this.setState((_a = {},
            _a[propName] = value,
            _a));
    };
    return ObservableComponent;
}(BaseComponent));
export { ObservableComponent };
