import * as tslib_1 from "tslib";
import "../../FabricInit";
import * as React from "react";
import { BaseComponent, css, format } from "office-ui-fabric-react/lib/Utilities";
import * as VSSUIResources from '../../VSS.Resources.VSSUI';
import "./FileInput.css";
/*
 * Make a best-effort attempt to determine the file encoding for the given base64 encoded file contents
 *
 * @param base64Content Base64 encoded string of the file contents
 */
function tryDetectFileEncoding(base64Content) {
    if (typeof window.atob === "undefined") {
        return 0 /* Unknown */;
    }
    var decodedContent = window.atob(base64Content);
    // Read the byte order mark
    var bom = [];
    for (var i = 0; i < 4; i++) {
        bom.push(decodedContent.charCodeAt(i));
    }
    if (bom[0] === 0xfe && bom[1] === 0xff) {
        return 6 /* UTF16_BE */;
    }
    else if (bom[0] === 0xff && bom[1] === 0xfe) {
        if (bom[2] === 0x00 && bom[3] === 0x00) {
            return 5 /* UTF32_LE */;
        }
        else {
            return 7 /* UTF16_LE */;
        }
    }
    else if (bom[0] === 0xef && bom[1] === 0xbb && bom[2] === 0xbf) {
        return 3 /* UTF8 */;
    }
    else if (bom[0] === 0x00 && bom[1] === 0x00 && bom[2] === 0xfe && bom[3] === 0xff) {
        return 4 /* UTF32_BE */;
    }
    var isAscii = true;
    // Check for binary files
    for (var i = 0, l = decodedContent.length; i < l; i++) {
        var charCode = decodedContent.charCodeAt(i);
        // If buffer[i] is 0-0x1F OR 0x7F but not (TAB, CR, LF, FF, or ^Z)
        // then the file is a binary file.
        if ((charCode <= 0x1f || charCode == 0x7f) &&
            (charCode != 0x9 && charCode != 0xc && charCode != 0xd && charCode != 0xa && charCode != 0x1a)) {
            return 1 /* Binary */;
        }
        else if (charCode > 0x7f) {
            isAscii = false;
        }
    }
    if (isAscii) {
        return 2 /* ASCII */;
    }
    // Treat non-binary files without an explicit BOM as UTF8
    return 3 /* UTF8 */;
}
var FileInput = /** @class */ (function (_super) {
    tslib_1.__extends(FileInput, _super);
    function FileInput() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._files = [];
        _this._clear = function () {
            _this._files.forEach(function (pendingResult) {
                if (pendingResult.fileReader.readyState !== FileReader.DONE) {
                    pendingResult.fileReader.abort();
                }
            });
            _this._files = [];
            _this.setState({
                files: _this._files,
                errorMessage: ""
            });
            _this._triggerUpdateEvent();
        };
        _this._remove = function (file) {
            _this._files = _this._files.filter(function (r) { return r !== file; });
            _this.setState({
                files: _this._files
            });
            _this._triggerUpdateEvent();
        };
        _this._addFiles = function (files) {
            // Make sure the file type selected is of the allowed file extenstion type
            if (_this.props.allowedFileExtensions && _this.props.allowedFileExtensions.length > 0) {
                for (var i = 0; i < files.length; i++) {
                    var fileExtension = files[i].name.split('.').pop();
                    var isAllowed = false;
                    for (var j = 0; j < _this.props.allowedFileExtensions.length; j++) {
                        if (fileExtension.toLocaleLowerCase() === _this.props.allowedFileExtensions[j].toLocaleLowerCase()) {
                            isAllowed = true;
                            break;
                        }
                    }
                    if (!isAllowed) {
                        var errorMessage = format(VSSUIResources.FileInputErrorNotAllowedExtensionType, fileExtension, _this.props.allowedFileExtensions.join());
                        _this.setState({
                            errorMessage: errorMessage
                        });
                        return;
                    }
                }
            }
            // Make sure the file names are unique, so build a hash set of names
            var fileNames = {};
            for (var i = 0, l = _this._files.length; i < l; i++) {
                fileNames[_this.props.fileNamesCaseSensitive ? _this._files[i].result.name : _this._files[i].result.name.toLowerCase()] = 1;
            }
            // Loop through each file checking size and name uniqueness
            var totalSize = _this._getTotalFilesSize();
            for (var i = 0, l = files.length; i < l; i++) {
                if (_this.props.maximumSingleFileSize && files[i].size > _this.props.maximumSingleFileSize) {
                    var errorMessage = _this._getLimitErrorMessage(format(VSSUIResources.FileInputErrorMaxFileSize, files[i].name, _this._getFriendlySizeString(files[i].size), _this._getFriendlySizeString(_this.props.maximumSingleFileSize)), { fileSize: files[i].size });
                    _this.setState({
                        errorMessage: errorMessage
                    });
                    return;
                }
                totalSize += files[i].size;
                var fileNameKey = _this.props.fileNamesCaseSensitive ? files[i].name : files[i].name.toLowerCase();
                if (fileNames[fileNameKey]) {
                    var errorMessage = format(VSSUIResources.FileInputErrorDuplicateFileName, files[i].name);
                    _this.setState({
                        errorMessage: errorMessage
                    });
                    return;
                }
                fileNames[fileNameKey] = 1;
            }
            // Check overall file size
            if (_this.props.maximumTotalFileSize && totalSize > _this.props.maximumTotalFileSize) {
                var errorMessage = _this._getLimitErrorMessage(format(VSSUIResources.FileInputErrorTotalFileSize, _this._getFriendlySizeString(totalSize), _this._getFriendlySizeString(_this.props.maximumTotalFileSize)), { totalSize: totalSize });
                _this.setState({
                    errorMessage: errorMessage
                });
                return;
            }
            // Check total number of files
            var fileCount = (_this._files.length + files.length);
            if (_this.props.maximumNumberOfFiles && fileCount > _this.props.maximumNumberOfFiles) {
                var message = _this.props.maximumNumberOfFiles === 1 ?
                    VSSUIResources.FileInputErrorSingleFileOnly :
                    format(VSSUIResources.FileInputErrorMaxNumFiles, _this.props.maximumNumberOfFiles);
                var errorMessage = _this._getLimitErrorMessage(message, { fileCount: fileCount });
                _this.setState({
                    errorMessage: errorMessage
                });
                return;
            }
            //
            // Add the files (error checks have passed)
            //
            for (var i = 0, l = files.length; i < l; i++) {
                _this._addFile(files[i]);
            }
        };
        return _this;
    }
    FileInput.prototype.render = function () {
        var maximumNumberOfFiles = this.props.maximumNumberOfFiles || 0;
        var files = this._files;
        var errorMessage = (this.state && this.state.errorMessage) ? this.state.errorMessage : "";
        var canAddMoreFiles = !maximumNumberOfFiles || (files.length < maximumNumberOfFiles);
        var accept = null;
        if (this.props.allowedFileExtensions) {
            var fileExtensions = this.props.allowedFileExtensions.map(function (extension) {
                if (extension[0] !== ".") {
                    return "." + extension;
                }
                else {
                    return extension;
                }
            });
            accept = fileExtensions.join();
        }
        var dragDropLabel = maximumNumberOfFiles === 1 ? VSSUIResources.FileInputDragDropSingleFileLabel : VSSUIResources.FileInputDragDropLabel;
        var multiple = maximumNumberOfFiles !== 1;
        return React.createElement(FileInputPure, { dragDropLabel: dragDropLabel, multiple: multiple, canAddMoreFiles: canAddMoreFiles, accept: accept, files: files, errorMessage: errorMessage, addFiles: this._addFiles, remove: this._remove, clear: this._clear });
    };
    FileInput.prototype._addFile = function (file) {
        var _this = this;
        //
        // Setup the pending result
        //
        var result = {
            name: file.name,
            size: file.size,
            type: file.type,
            lastModifiedDate: file.lastModifiedDate,
            statusText: VSSUIResources.Loading,
            isError: false
        };
        var fileReader = new FileReader();
        var fileInputData = {
            result: result,
            fileReader: fileReader
        };
        this._files.push(fileInputData);
        // Helper functions
        var _triggerUpdate = function () {
            _this.setState({
                files: _this._files,
                errorMessage: (_this.state && _this.state.errorMessage) ? _this.state.errorMessage : ""
            });
            _this._triggerUpdateEvent();
        };
        var _addResult = function () {
            result.statusText = _this._getFriendlySizeString(file.size);
            _triggerUpdate();
        };
        //
        // Setup the read operation
        //
        fileReader.onload = function (e) {
            result.encoding = 0 /* Unknown */;
            if (_this.props.resultContentType === 1 /* RawText */) {
                result.content = fileReader.result || "";
            }
            else {
                var dataUrl = fileReader.result;
                // If the file is empty, Chrome returns result="data:" and IE returns result=null, so we check for both of these cases.
                if (dataUrl && dataUrl.indexOf(",") > -1) {
                    // Get the base64 content portion of the data url
                    result.content = dataUrl.substr(dataUrl.indexOf(",") + 1);
                    if (result.content.substr(0, 2) === "//" && result.content.length % 4 === 2) {
                        // Trim a leading "//" only if it isn't part of the base64 content itself (base64 length is a multiple of 4).
                        result.content = result.content.substr(2);
                    }
                }
                else {
                    result.content = "";
                }
                if (_this.props.detectEncoding) {
                    result.encoding = tryDetectFileEncoding(result.content);
                }
            }
            _addResult();
        };
        fileReader.onerror = function (e) {
            var errorMessage = format(VSSUIResources.FileInputReadErrorFormat, result.name, fileReader.error.message);
            result.statusText = VSSUIResources.FileInputErrorLabel;
            result.isError = true;
            _triggerUpdate();
        };
        fileReader.onloadend = function (e) {
            _triggerUpdate();
        };
        //
        // Start the read operation
        //
        if (this.props.resultContentType === 1 /* RawText */) {
            fileReader.readAsText(file);
        }
        else if (this.props.resultContentType === 2 /* RawFile */) {
            result.encoding = 0 /* Unknown */;
            result.file = file;
            _addResult();
            _triggerUpdate();
        }
        else {
            fileReader.readAsDataURL(file);
        }
    };
    FileInput.prototype._triggerUpdateEvent = function () {
        if (this.props.updateHandler) {
            this.props.updateHandler({
                loading: this._files.some(function (file) {
                    return file.fileReader.readyState !== FileReader.DONE;
                }),
                files: this._files
            });
        }
    };
    FileInput.prototype._getTotalFilesSize = function () {
        var totalSize = 0;
        var files = this._files;
        for (var i = 0, l = files.length; i < l; i++) {
            totalSize += files[i].result.size;
        }
        return totalSize;
    };
    FileInput.prototype._getLimitErrorMessage = function (errorText, limitData) {
        var message = errorText;
        if (this.props.limitMessageFormatter) {
            message = this.props.limitMessageFormatter(errorText, limitData);
        }
        return message;
    };
    /**
     * Get a friendly string representing the length of a file (e.g. "61.2 KB")
     *
     * @param numBytes Number of bytes
     * @param decimalPlaces Number of decimal places to round to. (e.g. 0 -> "61 KB", 1 -> "61.2 KB", 2 -> "61.18 KB")
     */
    FileInput.prototype._getFriendlySizeString = function (numBytes, decimalPlaces) {
        if (decimalPlaces === void 0) { decimalPlaces = 1; }
        var divider = Math.pow(10, decimalPlaces);
        if (numBytes < 1024) {
            return format(VSSUIResources.FileSizeBytesFormat, numBytes);
        }
        else if (numBytes < (1024 * 1024)) {
            return format(VSSUIResources.FileSizeKBFormat, Math.round(numBytes / 1024 * 10) / 10);
        }
        else {
            return format(VSSUIResources.FileSizeMBFormat, Math.round(numBytes / (1024 * 1024) * 10) / 10);
        }
    };
    return FileInput;
}(BaseComponent));
export { FileInput };
var FileInputPure = /** @class */ (function (_super) {
    tslib_1.__extends(FileInputPure, _super);
    function FileInputPure() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._onClearClick = function () {
            if (_this.props.clear) {
                _this.props.clear();
            }
        };
        _this._onRemoveClick = function (file) {
            if (_this.props.remove) {
                _this.props.remove(file);
            }
        };
        _this._onRemoveKeyDown = function (event, file) {
            // Refer to https://developer.mozilla.org/en-US/docs/Web/API/KeyboardEvent/key/Key_Values
            if (event.key === "Enter" || event.key === " " || event.key === "Spacebar") {
                if (_this.props.remove) {
                    _this.props.remove(file);
                }
            }
        };
        _this._onBrowseContainerClick = function () {
            if (_this._fileInput) {
                _this._fileInput.click();
            }
        };
        _this._onBrowseContainerKeyPress = function (e) {
            if (_this._fileInput && (e.which === 13 /* enter */ || e.which === 32 /* space */)) {
                _this._fileInput.click();
            }
        };
        _this._resolveFileInput = function (element) {
            _this._fileInput = element;
        };
        _this._resolveRemoveLink = function (element) {
            if (!_this._removeLink) {
                _this._removeLink = element;
            }
        };
        _this._onDragEnter = function (e) {
            _this._setDropEffect(e);
            e.preventDefault();
            _this.setState({
                isDragging: true
            });
        };
        _this._onDragLeave = function (e) {
            e.preventDefault();
            _this.setState({
                isDragging: false
            });
        };
        _this._onDragOver = function (e) {
            _this._setDropEffect(e);
            e.preventDefault();
            _this.setState({
                isDragging: true
            });
        };
        _this._onDrop = function (e) {
            _this._setDropEffect(e);
            e.preventDefault();
            _this.setState({
                isDragging: false
            });
            var fileList = e.dataTransfer.files;
            if (fileList && fileList.length > 0) {
                if (_this.props.addFiles) {
                    _this.props.addFiles(fileList);
                }
            }
        };
        _this._onFileInputChange = function (args) {
            _this._removeLink = undefined;
            if (args.currentTarget.files && args.currentTarget.files.length) {
                if (_this.props.addFiles) {
                    _this.props.addFiles(args.currentTarget.files);
                }
                args.currentTarget.value = "";
            }
        };
        return _this;
    }
    FileInputPure.prototype.render = function () {
        var _this = this;
        // the file input is not shown. instead, we show a pretty drag-drop container with a browse button.
        var fileInputStyle = {
            display: "none"
        };
        var fileListStyle = {
            display: "block"
        };
        var dropTargetClassName = css("vss-FileInput-filesDropTarget", this.state.isDragging ? (this.props.dragOverCssClass || "vss-FileInput-dragOver") : "");
        return React.createElement("div", { className: "vss-FileInput" },
            this.props.canAddMoreFiles &&
                React.createElement("div", { className: dropTargetClassName, onDragOver: this._onDragOver, onDragLeave: this._onDragLeave, onDragEnter: this._onDragEnter, onDrop: this._onDrop },
                    React.createElement("div", { className: "vss-FileInput-dragDropTable", role: "presentation" },
                        React.createElement("div", { className: "vss-FileInput-dragDropLabel" },
                            React.createElement("span", null, this.props.dragDropLabel)),
                        React.createElement("div", { className: "vss-FileInput-browseButton" },
                            React.createElement("button", { className: "vss-FileInput-browseContainer", onKeyPress: this._onBrowseContainerKeyPress, onClick: this._onBrowseContainerClick }, VSSUIResources.FileInputBrowseButtonLabel),
                            React.createElement("input", { type: "file", ref: this._resolveFileInput, onChange: this._onFileInputChange, accept: this.props.accept, multiple: this.props.multiple, style: fileInputStyle })))),
            this.props.files.length > 1 &&
                React.createElement("div", { className: "vss-FileInput-overallStatusContainer" },
                    React.createElement("span", { className: "vss-FileInput-overallStatusActions" },
                        React.createElement("a", { onClick: this._onClearClick }, VSSUIResources.FileInputRemoveAll))),
            React.createElement("div", { className: "vss-FileInput-errorContainer" }, this.props.errorMessage),
            this.props.files.length > 0 &&
                React.createElement("ul", { className: "vss-FileInput-fileList", style: fileListStyle }, this.props.files.map(function (file) {
                    return React.createElement("li", { key: file.result.name },
                        React.createElement("div", { className: "vss-FileInput-headerContainer" },
                            React.createElement("span", { className: "vss-FileInput-fileName" }, file.result.name)),
                        React.createElement("div", { className: "vss-FileInput-detailsContainer" },
                            React.createElement("span", { className: "vss-FileInput-status" }, file.result.statusText),
                            React.createElement("span", { className: "vss-FileInput-links" },
                                React.createElement("a", { tabIndex: 0, ref: _this._resolveRemoveLink, onKeyDown: function (event) { return _this._onRemoveKeyDown(event, file); }, onClick: function () { return _this._onRemoveClick(file); }, role: "button" }, VSSUIResources.FileInputRemoveFile))));
                })));
    };
    FileInputPure.prototype.componentWillMount = function () {
        this.setState({
            isDragging: false
        });
    };
    FileInputPure.prototype.componentDidUpdate = function () {
        if (this._removeLink) {
            this._removeLink.focus();
        }
    };
    FileInputPure.prototype._setDropEffect = function (e) {
        /*
         * React wraps event handlers inside other events in the development build to get better devtool integration.
         * Problem is that Edge and IE don't allow access to the dropEffect property unless you're inside that original event handler.
         * In the production build there are no wrapper events, it's just a function call, so dropEffect can be accessed normally.
         *
         * See https://github.com/facebook/react/issues/5700
         */
        try {
            e.dataTransfer.dropEffect = "copy";
        }
        catch (ex) {
            /*
             * So, we just ignore the exception here.
             */
        }
    };
    return FileInputPure;
}(BaseComponent));
