export interface ILayer {

}
/**
 * The LayerSize is used to control how large the layer is on its oriented axis.
 * So a vertical layer will define its width and a horizontal will define its
 * height. The secondary axis will be sized by the content unless Stretch is used
 * in which case it is 100% the size of the container.
 */
export const enum LayerSize {

    /**
     * The small layer is 240px wide.
     */
    Small = 0,

    /**
     * The medium layer is 320px wide.
     */
    Medium = 1,

    /**
     * The large layer is 400px wide.
     */
    Large = 2,

    /**
     * A custom size means the size of the element is either based on the contents
     * or any custom css applied through the contentClassName.
     */
    Custom = 255
}

/**
 * Within the layer is the content justified to the start, center, or end.
 */
export const enum LayerJustification {

    /**
     * Justify the content element to the start of the layer.
     */
    Start = 0,

    /**
     * Justify he content element at the center of the layer.
     */
    Center = 1,

    /**
     * Justify the content element at the end of the layer.
     */
    End = 2,

    /**
     * The content element should stretch to the length of the layer (from start to end).
     */
    Stretch = 3,

    /**
     * No justification is applied to the content element.
     */
    Custom = 255
}

/**
 * LayerLocation is used to describe where the content of the layer should be shown.
 * This supports Start and End which in a LeftToRight UI Start = left and End = right.
 */
export const enum LayerLocation {

    /**
     * The location of start is based on the LTR layout of the page. 
     * The default left to right has Start showing up on the left.
     */
    Start = 0,

    /**
     * Center the content within the layer.
     */
    Center = 1,

    /**
     * The location of end is based on the LTR layout of the page. 
     * The default left to right has End showing up on the right.
     */
    End = 2,

    /**
     * With a custom location, no default location is given to the content.
     * The caller should apply classes to the layer/layer content to position
     * and size the children.
     */
    Custom = 255
}

/**
 * The LayerOrientation defines how the contents are generally oriented.
 */
export const enum LayerOrientation {

    /**
     * The contents should be oriented as a vertical column.
     */
    Vertical = 0,

    /**
     * The contents should be oriented as a horizontal row.
     */
    Horizontal = 1,

    /**
     * No specified orientation is specifed and the caller will apply the required
     * classes to orient the layer and its contents.
     */
    Custom = 255
}

/**
 * A Layer Component is used to create an absolutely positioned filler for the parent component
 * that generally gives you a modal or modeless look and feel. The caller then places their
 * custom elements within the layer. This can be used for things like dialogs, panels, etc.
 */
export interface ILayerProps {

    /**
     * Optional callback to access the ILayer interface. Use this instead of ref for accessing
     * the public methods and properties of the component.
     */
    componentRef?: (component: ILayer | null) => void;

    /**
     * Label used to describe the contents of the layer for aria enabled devices.
     */
    ariaLabel?: string;

    /**
     * Custom classes that are applied to the root element of the layer.
     */
    className?: string;

    /**
     * Custom classes that are applied to the content element of the layer.
     * The callers content will be a direct child of the content element.
     */
    contentClassName?: string;

    /**
     * Location the content should be placed within the layer.
     */
    contentLocation?: LayerLocation;

    /**
     * Justification of the content element.
     */
    contentJustification?: LayerJustification;

    /**
     * Orientation the content should be placed within the layer.
     */
    contentOrientation?: LayerOrientation;

    /**
     * Should a default shadow be shown around the layer content.
     */
    contentShadow?: boolean;

    /**
     * Size of the layers primary axis.
     */
    contentSize?: LayerSize;

    /**
     * DOM element id
     */
    id?: string;

    /**
     * If the caller wants the layer to "dismiss" when an unhandled esc key is used
     * within the layer content.
     */
    isEscDismiss?: boolean;
    
    /**
     * If the caller wants the layer to "dismiss" when a mouse click is performed outside
     * the layer content, or focus leaves the layer contents children.
     */
    isLightDismiss?: boolean;

    /**
     * This will ensure a semi-transparent view of the layered element.
     */
    isModal?: boolean;

    /**
     * The element that opened the layer. If the opening element is supplied to the layer, the
     * layer will ignore lightDismiss requests against this element. The openingElement can
     * be either a raw HTMLElement, or a selector.
     */
    openingElement?: HTMLElement | string;

    /**
     * The onDismiss method is called when the layer is dismissed.
     */
    onDismiss?: () => void;
}