import * as tslib_1 from "tslib";
import "../../Core/core.scss";
import "../../Common.scss";
import * as React from "react";
import { ObservableLike } from '../../Core/Observable';
/**
 * Handles subscription to properties that are IObservableValues, so that components don't have to handle on their own.
 *
 * Usage:
 *
 * <Observer myObservableValue={observableValue} className='foo'>
 *     <MyComponent myObservableValue='' />
 * </Observer>
 *
 * Your component will get re-rendered with the new value of myObservableValue whenever that value changes.
 * Additionally, any additional props set on the Observer will also get passed down.
 */
var Observer = /** @class */ (function (_super) {
    tslib_1.__extends(Observer, _super);
    function Observer(props) {
        var _this = _super.call(this, props) || this;
        _this.subscriptions = {};
        // Initialize the state with the initial value of the observable.
        var state = {};
        for (var propName in _this.props) {
            state[propName] = ObservableLike.getValue(_this.props[propName]);
        }
        _this.state = state;
        return _this;
    }
    Observer.prototype.render = function () {
        var child = React.Children.only(this.props.children);
        var newProps = {};
        // Copy over any properties from the observable component to the children.
        for (var key in this.state) {
            if (key !== "children") {
                newProps[key] = this.state[key];
            }
        }
        return React.cloneElement(child, newProps, child.props.children);
    };
    Observer.prototype.componentDidMount = function () {
        // Subscribe to the each of the properties for changes.
        var stateUpdates = {};
        var stateChanged = false;
        for (var propName in this.props) {
            this._subscribe(propName, this.props);
            // Look for changes in the observables between creation and now.
            if (this.state[propName] !== this.props[propName]) {
                stateUpdates[propName] = ObservableLike.getValue(this.props[propName]);
                stateChanged = true;
            }
        }
        // If any state updates occurred update the state now.
        if (stateChanged) {
            this.setState(stateUpdates);
        }
    };
    Observer.prototype.componentWillUnmount = function () {
        // Unsubscribe from any of the observable proeprties.
        for (var propName in this.props) {
            this._unsubscribe(propName, this.props);
        }
    };
    Observer.prototype.componentWillReceiveProps = function (nextProps) {
        var oldProps = this.props;
        var newState = tslib_1.__assign({}, this.state);
        var stateChanged = false;
        for (var propName in oldProps) {
            var oldValue = oldProps[propName];
            var newValue = nextProps[propName];
            if (oldValue !== newValue) {
                this._unsubscribe(propName, this.props);
                if (newValue !== undefined) {
                    this._subscribe(propName, nextProps);
                    newState[propName] = ObservableLike.getValue(newValue);
                }
                else {
                    delete newState[propName];
                }
                stateChanged = true;
            }
        }
        for (var propName in nextProps) {
            var oldValue = oldProps[propName];
            if (oldValue === undefined) {
                var newValue = nextProps[propName];
                this._subscribe(propName, nextProps);
                newState[propName] = ObservableLike.getValue(newValue);
                stateChanged = true;
            }
        }
        if (stateChanged) {
            this.setState(newState);
        }
    };
    Observer.prototype._subscribe = function (propName, props) {
        var observableValue = props[propName];
        if (ObservableLike.isObservable(observableValue)) {
            var delegate = this._onValueChanged.bind(this, propName);
            ObservableLike.subscribe(observableValue, delegate);
            this.subscriptions[propName] = delegate;
        }
    };
    Observer.prototype._unsubscribe = function (propName, props) {
        var observableValue = this.props[propName];
        if (ObservableLike.isObservable(observableValue)) {
            var delegate = this.subscriptions[propName];
            ObservableLike.unsubscribe(observableValue, delegate);
            delete this.subscriptions[propName];
        }
    };
    Observer.prototype._onValueChanged = function (propName, value, action) {
        var _a;
        this.setState((_a = {},
            _a[propName] = value,
            _a));
    };
    return Observer;
}(React.Component));
export { Observer };
