import "../../FabricInit";
import * as React from "react";
import { ITextField, ITextFieldProps as IFabricTextField } from "office-ui-fabric-react/lib/TextField";

export interface ITextField extends IFabricTextField {

}

export interface ITextFieldProps extends React.AllHTMLAttributes<HTMLInputElement | HTMLTextAreaElement> {
    /**
     * Optional callback to access the ITextField interface. Use this instead of ref for accessing
     * the public methods and properties of the component.
     */
    componentRef?: (component: ITextField | null) => void;

    /**
     * Whether or not the textfield is a multiline textfield.
     * @default false
     */
    multiline?: boolean;

    /**
     * Whether or not the multiline textfield is resizable.  Cannot be on a field with autoAdjustHeight enabled.
     * @default false
     */
    resizable?: boolean;

    /**
     * Whether or not to auto adjust textField height. Applies only to multiline textfield.
     * @default true
     */
    autoAdjustHeight?: boolean;

    /**
     * Optional custom renderer for the label.
     */
    onRenderLabel?: IRenderFunction<ITextFieldProps>;

    /**
     * The textfield input description
     */
    description?: string;

    /**
     * Optional custom renderer for the description.
     */
    onRenderDescription?: IRenderFunction<ITextFieldProps>;

    /**
    * Custom render function for prefix.
    */
    onRenderPrefix?: IRenderFunction<ITextFieldProps>;

    /**
    * Custom render function for suffix.
    */
    onRenderSuffix?: IRenderFunction<ITextFieldProps>;

    /**
     * Optional icon props for a prefix icon.
     */
    prefixIconProps?: IVssIconProps;

    /**
     * Optional icon props for a suffix icon.
     */
    suffixIconProps?: IVssIconProps;

    /**
     * Render with larger text and height
     */
    textFieldSize?: TextFieldSize;

    /**
     * Default value of the textfield, if any. Only provide this if the textfield is an uncontrolled component;
     * otherwise, use the "value" property.
     */
    defaultValue?: string;

    /**
     * Current value of the textfield. Only provide this if the textfield is a controlled component where you
     * are maintaining its current state; otherwise, use the "defaultValue" property.
     */
    value?: string;

    /**
     * Disabled state of the textfield.
     * @default false
     */
    disabled?: boolean;

    /**
     * Callback for the onChanged event.
     */
    onChanged?: (newValue: any) => void;

    /**
     * Callback for the onBeforeChange event.
     */
    onBeforeChange?: (newValue: any) => void;

    /**
     * Callback for when the input value changes.
     */
    onChange?: (event: React.FormEvent<HTMLInputElement | HTMLTextAreaElement>, newValue?: string) => void;

    /**
     * Aria Label for textfield, if any.
     */
    ariaLabel?: string;

    /**
     * Whether the input field should have autocomplete enabled.
     * This tells the browser to display options based on earlier typed values.
     */
    autoComplete?: 'on' | 'off';

    /**
     * Determines when and how the value of the text box is validated
     */
    validator?: Validator;
}

export interface IValidator {

    /**
     * A function that takes in a value and returns an error message if it does not meet the validation requirements. 
     * Undefined is return if it meets them.
     */
    validate: (value: string) => string | undefined;
    /**
     * Run validation only on input focus
     * @default false
     */
    validateOnFocusIn?: boolean;

    /**
     * Run validation only on input focus out
     * @default false
     */
    validateOnFocusOut?: boolean;

    /**
     * Optional flag to disable onload validation
     * @default true
     */
    validateOnLoad?: boolean;
    /**
     * Text field will start to validate after users stop typing for `deferredValidationTime` milliseconds.
     * @default 200
     */
    deferredValidationTime?: number;
    /**
     * Callback for the onNotifyValidationResult event.
     */
    onNotifyValidationResult?: (errorMessage: string, value: string | undefined) => void;
}

export const enum TextFieldSize {
    /**
     * 32px tall with 14px text
     */
    Normal = "normal",
    /**
     * 48px tall with 17px text
     */
    Large = "large",
}