import * as React from "react";
import { IColumn } from "OfficeFabric/DetailsList"
import { SelectionMode } from "OfficeFabric/Selection"

export interface ITreeList<T> {
    /**
     * 
     * @param selectedItemIds Set selected item ids in the TreeList. If the items are not yet there, select those items when they
     * are added to the TreeList. User manual selection cancels this action.
     */
    setSelectedItemsById(selectedItemIds: string[]);
}

export interface ITree<T> {
    /**
     * 
     * @param selectedItemIds Set selected item ids in the Tree. If the items are not yet there, select those items when they
     * are added to the Tree. User manual selection cancels this action.
     */
    setSelectedItemsById(selectedItemIds: string[]);
}

export interface ITreeListItem<T> {
    /** 
     * A reference back to the ITreeItem for this list item. 
     */
    treeItem: ITreeItem<T>;

    /** 
     * How many levels deep we should indent this row. 
     */
    indentLevel: number;

    /** 
     * Whether or not the item is able to expanded/collapsed. 
     */
    isExpandable: boolean;

    /** 
     * Whether or not the item is currently expanded - defaults to false. 
     */
    isExpanded?: boolean;

    /** 
     * Whether or not the item is currently loading children - defaults to false. 
     */
    isLoading?: boolean;
}

export interface ITreeItem<T> {
    /** 
     * A unique identifier across all items in the tree. 
     */
    id: string;

    /** 
     * An optional reference back to the original item wrapped by the tree item. 
     */
    item?: T;

    /** 
     * Whether or not the item is able to expanded/collapsed. 
     */
    isExpandable: boolean;

    /**
     * The initial expansion state for this item when the tree is first rendered. Changing this value on subsequent renders will have no effects.
     */
    defaultIsExpanded?: boolean;
}

interface ISharedTreeProps<T> {

    /**
     * Columns to draw in the tree. Currently must have an onRender which will be wrapped.
     */
    columns: IColumn[];

    /**
     * Which column should contain the collapse/expand behavior for the tree. If not provided, defaults to the first column.
     */
    primaryTreeColumnKey?: string;

    /**
     * Optional custom className for the tree.
     */
    className?: string;

    /**
     * Additional custom keyboard behaviour
     */
    onRowKeyDown?:(event: React.KeyboardEvent<HTMLDivElement>, item: T) => void;

     /**
     * overriding custom single click behaviour on primary cell instead of expand or collapse
     */
    onPrimaryCellClick?: (event: React.MouseEvent<HTMLDivElement>, item: T) => void;

    ////////////////////////////////
    // IVssDetailsListProps below //
    ////////////////////////////////

    /**
     * Which column should contain the actions button trigger
     */
    actionsColumnKey?: string;

    /**
     * Whether or not the actions button should use display: none or visibility: hidden while it is not visible.
     */
    allocateSpaceForActionsButtonWhileHidden?: boolean;

    /**
     * A callback used to dynamically create the list of menu items
     */
    getMenuItems?: (activatedItem: any) => IContextualMenuItem[];

    /**
     * A callback used to dynamically create the list of menu item providers.  These can be used to asynchronously load menu items.
     */
    getMenuItemProviders?: (activatedItem: any) => IVssContextualMenuItemProvider[];

    /**
     * Called when the selected items in the TreeList change
     */
    onSelectedItemsChanged?: (selectedItems: T[]) => void;

    /**
     * Single-selection mode or none for the TreeList
     */
    selectionMode?: SelectionMode;

    /**
     * Show column header or not
     */
    isHeaderVisible?: boolean = true;

    /**
     * The initial focused index
     */
    initialFocusedIndex?: number

    /** 
     * Callback for when a given row has been mounted. Useful for identifying when a row has been rendered on the page. 
     * */
    onRowDidMount?: (item?: any, index?: number) => void;
}

export interface ITreeListProps<T> extends ISharedTreeProps<T> {
    /**
     * The list of all items to be rendered in the tree list. 
     */
    treeListItems: ITreeListItem<T>[];

    /**
     * Called whenever an item is expanded or collapsed. The isExpanded value on the treeListItem is the current 
     * (pre-expansion/collapse) state of the item.
     */
    onItemToggled: (treeListItem: ITreeListItem<T>) => void;

    /**
     * Optional callback to access the ITreeList interface. Use this instead of ref for accessing
     * the public methods and properties of the component.
     */
    componentRef?: (component: ITreeList<T>) => void;
}

export interface ITreeProps<T> extends ISharedTreeProps<T> {
    /**
     * Optional callback to access the ITree interface. Use this instead of ref for accessing
     * the public methods and properties of the component.
     */
    componentRef?: (component: ITree<T>) => void;

    /**
     * Item provider used to build the tree data structure. Every new instance of this will build the tree from root, send the same instance if items didn't change.
     */
    treeItemProvider: ITreeItemProvider<T>;
}

export interface ITreeItemProvider<T> {
    /**
     * For the provided item, returns an array of items, or a Promise of an array of items representing
     * the children of that item.
     * 
     * If item is undefined, the root items of the tree should be returned. 
     */
    getChildren: (item: T | undefined) => T[] | Promise<T[]>;

    /**
     * Defines how to convert an item of the user's type into an ITreeItem
     */
    getTreeItem: (item: T) => ITreeItem<T>;

    getTreeListItem: (item: T) => ITreeListItem<T>;

    setTreeListItem: (item: T, treeListItem: ITreeListItem<T>) => void;
}