import "../../FabricInit";
import * as React from "react";
import { IBreadCrumbData, IBreadcrumbItem } from "office-ui-fabric-react/lib/Breadcrumb";
import { IBaseProps } from "office-ui-fabric-react/lib/Utilities";
import { IItemIndicatorProps } from '../../ItemIndicator';
import { IItemPickerProvider, IPickListProps, IPickListItem, IPickListGroup, IPickListAction, IPickListIndicator, IPickListSelection } from '../../PickList';
import { IVssIconProps } from '../../VssIcon';
import { IObservableValue } from '../../Core/Observable';
import { IBreadCrumbData } from './Breadcrumb.base';
import { IIconProps } from 'office-ui-fabric-react/lib/Icon';
import { IRenderFunction, IStyleFunction, IComponentAs } from '../../Utilities';
import { IBreadcrumbStyleProps, IBreadcrumbStyles } from './Breadcrumb.styles';
import { ITheme } from '../../Styling';
import { IItemProviderWithCallback } from '../../Utilities/ItemContribution';
/**
 * Types of dividers that appear between segments of the breadcrumb
 */
export const enum VssBreadcrumbDividerType {
    /**
     * Right chevron for normal items, down chevron for items with picklists
     */
    Chevron = 0,
    /**
     * /
     */
    ForwardSlash = 1
}

export const enum VssBreadcrumbStyles {
    empty = 0x0,
    smallSize = 0x1,
    lightColor = 0x2,
    extraPadding = 0x4,
    dividerLink = 0x8,
    minHeight = 0x10
}

export interface IVssBreadcrumbItemProvider extends IItemProviderWithCallback<IVssBreadcrumbItem> {
}

export interface IVssBreadcrumbBaseProps extends IBaseProps {
    /**
     * Optional index where overflow items will be collapsed. Defaults to 0.
     */
    overflowIndex?: number;

    /**
     * Method to call when reducing the length of the breadcrumb.
     * Return undefined to never reduce breadcrumb length
     */
    onReduceData?: (data: IBreadCrumbData) => IBreadCrumbData | undefined;

    /**
     * Optional callback to be called when the resize group renders data.
     */
    dataDidRender?: (renderedData: any) => void;

    /**
     * The maximum number of breadcrumbs to display before coalescing.
     * If not specified, all breadcrumbs will be rendered.
     */
    maxDisplayedItems?: number;
}

export interface IVssBreadcrumbProps extends IVssBreadcrumbBaseProps {
    /**
     * items to display
     * @default []
     */
    items?: IVssBreadcrumbItemInternal[];

    /**
     * Max width for the bradcrumb item
     */
    maxItemWidth?: string;

    /**
     * Class name to use for the pick list for the header item picker dropdown
     */
    pickListClassName?: string;

    /**
     * Fixed width for the header item picker dropdown, in pixels
     */
    pickListWidth?: number;

    /**
     * Min width for the header item picker dropdown, in pixels
     */
    pickListMinWidth?: number;

    /**
     * Max width for the header item picker dropdown, in pixels
     */
    pickListMaxWidth?: number;

    /**
     * Max-height for the item picker dropdown, in pixels
     */
    pickListMaxHeight?: number;

    /**
     * Aria label to be used for the entire breadcrumb
     */
    ariaLabel?: string;

    /**
     * Type of divider between breadcrumb items
     * @default VssBreadcrumbDivider.Chevron
     */
    dividerType?: VssBreadcrumbDividerType;

    /**
     * Styles to apply to the breadcrumb items/divider.
     */
    presentationStyles?: VssBreadcrumbStyles;

    /**
     * Class name to apply to the root element
     */
    className?: string;

    /**
     * Optional prop to indicate whether item focus should be managed with a focus zone.
     * @default false
     */
    shouldUseFocusZone?: boolean;

    /**
     * Providers of breadcrumb items.
     */
    providers?: IVssBreadcrumbItemProvider[];

    /**
     * Optional callback to call when all contributed items have been loaded or some have loaded and the
     * loading timeout has been reached .
     * @default () => {}
     */
    onItemsLoaded?: () => void;
}

export interface IPickerDropdownProps extends IPickListProps {

    /**
     * Breadcrumb item to use for showing the picker dropdown.
     */
    item: IVssBreadcrumbItemInternal;

    /**
     * Element that the callout will pin to.
     */
    getPinElement: () => HTMLElement;

    /**
     * width for the PickList used in the dropdown
     */
    width: number;

    /**
     * Min width for the PickList. 
     */
    minWidth?: number;

    /**
     * Max width for the PickList. 
     */
    maxWidth?: number;

    /**
     * max-height for the picklist used in the dropdown
     */
    maxHeight?: number;

    /**
     * Class name to use for the pick list for the header item picker dropdown
     */
    pickListClassName?: string;

    /**
     * Called when the Picker Dropdown is opened
     */
    onPickListOpened?: () => void;

    /**
     * Called when the Picker Dropdown is closed
     */
    onPickListClosed?: () => void;

    /**
     * Called when the mouse enters the Picker Dropdown
     */
    onPickListMouseEnter?: (ev) => any;

    /**
     * Called when the mouse leaves the Picker Dropdown
     */
    onPickListMouseLeave?: (ev) => any;
}

export interface IVssBreadcrumbItem extends IBreadcrumbItem {

    /**
     * Props for an icon that is displayed on the left side of the breadcrumb
     */
    leftIconProps?: IVssIconProps;

    /**
     * Aria label to be used for an individual breadcrumb item
     */
    ariaLabel?: string;

    /**
     * Specifies behavior for the PickList that can be shown next to each breadcrumb item.
     */
    itemPicker?: IItemPickerProvider;

    /**
     * Optional function to specify custom rendering of item in PickList
     */
    onRender?: (item: IPickListItem) => JSX.Element;

    /**
     * Specifies whether the breadcrumb is a header title item.
     */
    isHeaderPickerItem?: boolean;

    /**
     * Optional number for the width of the picker for this item.
     */
    pickerWidth?: number;

    /**
     * Optional class for the picker for this item.
     */
    pickListClassName?: string;

    /**
     * Sets the order items should be displayed in (lowest first, undefined and 0 last).
     */
    rank?: number;

    /**
     * If multiple items with the same id are contributed, the one with the highest priority will be shown.
     */
    priority?: number;

    /**
     * Aria description for an individual breadcrumb item
     */
    ariaDescription?: string;
}

export interface IVssBreadcrumbItemInternal extends IVssBreadcrumbItem {
    /**
     * Callback that is called when the selection of the PickList changes
     * 
     * @returns True to prevent the default handler for the event that triggered the change.
     */
    onSelectionChanged?: (selection: IPickListSelection) => void | boolean;

    /**
     * If true, do not show the item picker and make breadcrumb non-clickable.
     */
    nonInteractive?: boolean;
}

export interface IVssBreadcrumb {
    /**
     * Breadcrumb pick list if supplied
     */
    pickList?: IVssBreadcrumbPickList;
}

export interface IVssBreadcrumbPickList {
    /**
     * Open the pick list dropdown
     */
    openDropdown(): void;

    /**
     * Close the pick list dropdown
     */
    closeDropdown(): void;

    /**
     * Toggles the pick list dropdown - opens it if it is currently closed,
     * and closes it if it is currently empty
     */
    toggleDropdown(): void;
}

export interface IInternalBreadcrumbProps {
    /**
     * Optional callback to access the IBreadcrumb interface. Use this instead of ref for accessing
     * the public methods and properties of the component.
     */
    componentRef?: (component: IBreadcrumb | null) => void;

    /**
     * Collection of breadcrumbs to render
     */
    items: IBreadcrumbItem[];

    /**
     * Optional root classname for the root breadcrumb element.
     */
    className?: string;

    /**
     * Render a custom divider in place of the default chevron '>'
     */
    dividerAs?: IComponentAs<IDividerAsProps>;

    /**
     * The maximum number of breadcrumbs to display before coalescing.
     * If not specified, all breadcrumbs will be rendered.
     */
    maxDisplayedItems?: number;

    /** Method to call when trying to render an item. */

    onRenderItem?: IRenderFunction<IBreadcrumbItem>;

    /**
     * Method to call when reducing the length of the breadcrumb.
     * Return undefined to never reduce breadcrumb length
     */
    onReduceData?: (data: IBreadCrumbData) => IBreadCrumbData | undefined;

    /**
     * Optional callback to be called when the resize group renders data.
     */
    dataDidRender?: (renderedData: any) => void;

    /**
     * Aria label to place on the navigation landmark for breadcrumb
     */
    ariaLabel?: string;

    /**
     * Optional name to use for aria label on overflow button.
     */
    overflowAriaLabel?: string;

    /**
     * Optional index where overflow items will be collapsed. Defaults to 0.
     */
    overflowIndex?: number;

    /**
     * Optional prop to indicate whether item focus should be managed with a focus zone.
     * @default true
     */
    shouldUseFocusZone?: boolean;
}

export interface IInternalBreadcrumbItem {
    /**
     * Text to display to the user for the breadcrumb
     */
    text: string;

    /**
     * Arbitrary unique string associated with the breadcrumb
     */
    key: string;

    /**
     * Callback issued when the breadcrumb is selected.
     */
    onClick?: (ev?: React.MouseEvent<HTMLElement>, item?: IBreadcrumbItem) => void;

    /**
     * Url to navigate to when this breadcrumb is clicked.
     */
    href?: string;
}

export interface IDividerAsProps extends IIconProps {
    /**
     * Optional breadcrumb item corresponds to left of the divider to be passed for custom rendering.
     * For overflowed items, it will be last item in the list
     */
    item?: IBreadcrumbItem;
}