/**
 * Core client interface for an IObservable collection keyed by K with values of type V.
 */
export interface IObservable<T, TAction extends string = string> {
    /**
     * subscribe should be called when the caller wants to be notified about changes to
     * the underlying data. The caller should only call once per delegate, but will
     * get notified N times (once for each call to subscribe).
     *
     * @param observer - This is the delegate to be notified when the underlying data changes.
     *
     * @param action - Optional argument that allows the consumer to supply a action
     *  with the delegate. If the action is supplied only those actions are delievered,
     *  while all actions are delivered is no action is supplied.
     */
    subscribe: (observer: (value: T, action?: TAction) => void, action?: TAction) => void;
    /**
     * unsubscribe should be called with a previously supplied delegate to subscribe.
     * The client MUST call unsubscribe once for every call to subscribe with the
     * appropriate delegates.
     *
     * @param observer - This is the delegate that was previously registered with subscribe.
     *
     * @param action - Optional argument that defines the action that was subscribed to.
     */
    unsubscribe: (observer: (value: T, action?: TAction) => void, action?: TAction) => void;
}
/**
 * An Observable implementation that will track a set of subscribers and supports
 * notifications when the underlying system changes.
 */
export declare class Observable<T, TAction extends string = string> implements IObservable<T, TAction> {
    private observers;
    private events?;
    /**
     * notify is used to send the event to all subscribers that have signed up for this events
     * action. This means they have subscribed directly to this action, or to all actions.
     * If the caller requested the event be persisted the event will be fired in order to new
     * subscribers as well when they subscribe.
     *
     * @param value - The object that represents the event data.
     *
     * @param action - The action that happened on this observable to produce the event.
     *
     * @param persistEvent - Optional value that determines if all future subscribers will
     *  recieve the event as well.
     */
    notify(value: T, action: TAction, persistEvent?: boolean): void;
    subscribe(observer: (value: T, action: TAction) => void, action?: string): void;
    unsubscribe(observer: (value: T, action: TAction) => void, action?: string): void;
}
export declare type IObservableLikeValue<T> = IObservableValue<T> | T;
export declare namespace ObservableLike {
    /**
     * Check whether the specified object is an observable or not.
     *
     * @param observableLike Object to perform observable check.
     */
    function isObservable<T>(observableLike: IObservable<T> | any): boolean;
    /**
     * Gets the value of the specified observable like. If not observable, returns the passed argument.
     *
     * @param observableLike Object to get the value.
     * @returns Observable value or the observable like itself.
     */
    function getValue<T>(observableLike: IObservableLikeValue<T>): T;
    /**
     * Subscribes to the specified object if it is an observable.
     *
     * @param observableLike Object to subscribe its value change if applicable.
     * @param observer Delegate to be executed when the underlying data changes.
     * @param action Optional argument that allows the consumer to supply a action
     *  with the delegate. If the action is supplied only those actions are delievered,
     *  while all actions are delivered is no action is supplied.
     */
    function subscribe<T>(observableLike: IObservable<T> | any, observer: (value: T, action: string) => void, action?: string): void;
    /**
     * Unsubscribes from the specified object if it is an observable.
     *
     * @param observableLike Object to subscribe its value change if applicable.
     * @param observer Delegate to be executed when the underlying data changes.
     * @param action Optional argument that allows the consumer to supply a action
     *  with the delegate. If the action is supplied only those actions are delievered,
     *  while all actions are delivered is no action is supplied.
     */
    function unsubscribe<T>(observableLike: IObservable<T> | any, observer: (value: T, action: string) => void, action?: string): void;
}
/**
 * An IReadonlyObservableValue<T> gives a readonly view of an IObservableValue<T>.
 *
 * The normal pattern to follow is for a parent object/component creates an IObservableValue<T>
 * and pass it to dependants as an IReadonlyObservableValue<T>. This prevents the callee
 * from changing the value and treating the relationship as a two way binding. Observables
 * are intended to be used as a one way binding where the object owner uses the observable to
 * notify others about changes to the value without giveing them control over the value.
 */
export interface IReadonlyObservableValue<T> extends IObservable<T> {
    /**
     * Read access to the value being observed.
     */
    readonly value: T;
}
/**
 * An IObservableValue<T> tracks an instance of type T and allows consumers
 * be notified with the value is changed.
 *
 * EventTypes:
 *  set - T
 */
export interface IObservableValue<T> extends IReadonlyObservableValue<T> {
    /**
     * This is the current value of the observable.
     */
    value: T;
}
export declare class ObservableValue<T> extends Observable<T> implements IObservableValue<T> {
    private v;
    constructor(value: T);
    value: T;
}
/**
 * When an action occurs on an IObservableCollection the event should take the form
 * of an ICollectionEntry<T> where T is the type of object being stored in
 * the collection.
 */
export interface ICollectionEntry<T> {
    key: string;
    value?: T;
}
/**
 * An Observable collection is used to track a set of objects by name and offer notifications
 * for consumers when the collection has changed.
 *
 * EventTypes:
 *  add - ICollectionEvent<V>
 */
export interface IObservableCollection<V> extends IObservable<ICollectionEntry<V>> {
    /**
     * Adding an object to the collection will notify all observers of the collection
     * and keep track of the objects.
     *
     * @param objectName - name of the object be registered.
     *
     * @param objectDefinition - details of the object being registered
     */
    add: (objectName: string, objectDefinition: V) => void;
    /**
     * get is used to retrieve the objectDefinition for named object.
     *
     * @param objectName - name of the object to get the definition.
     */
    get: (objectName: string) => V | undefined;
    /**
     * A read-only collection of the existing objects.
     */
    keys: () => string[];
}
/**
 * An ObservableCollection can be used to key a named collection of objects
 * and offer an observable endpoint.
 */
export declare class ObservableCollection<V> extends Observable<ICollectionEntry<V>> implements IObservableCollection<V> {
    private objects;
    add(objectName: string, objectDefinition: V): void;
    get(objectName: string): V | undefined;
    keys(): string[];
}
/**
 * All ObservableArray events will have an action associated with them.
 */
export interface IObservableArrayEventArgs<T> {
    /**
     * Items added to ObservableArray.
     */
    addedItems?: T[];
    /**
     * Items removed from ObservableArray.
     */
    removedItems?: T[];
}
/**
 * An Observable array is used to track an array of items and offer notifications
 * for consumers when the array has changed.
 *
 * EventTypes:
 *  push - { addedItems }
 *  pop - { removedItems}
 *  splice - { addedItems, removedItems }
 */
export interface IReadonlyObservableArray<T> extends IObservable<IObservableArrayEventArgs<T>> {
    /**
     * Returns the index of the first occurrence of a value in ObservableArray.
     *
     * @param searchElement  - The value to locate in the array.
     *
     * @param fromIndex  - The array index at which to begin the search. If fromIndex is omitted, the search starts at index 0.
     *
     * @returns The index of the searched value. If not found, returns 0.
     */
    indexOf(searchElement: T, fromIndex?: number): number;
    /**
     * Gets all the items in ObservableArray.
     */
    readonly value: T[];
    /**
     * Gets the number of the items in ObservableArray.
     */
    length: number;
}
/**
 * WARNING: A copy of this interface also exists in WebPlatform\Web\extensions\vss-web\vss-platform\Context.ts
 * Please make sure that you keep both instances in sync.
 *
 * An Observable array is used to track an array of items and offer notifications
 * for consumers when the array has changed.
 *
 * EventTypes:
 *  push - { addedItems }
 *  pop - { removedItems}
 *  splice - { addedItems, removedItems }
 */
export interface IObservableArray<T> extends IReadonlyObservableArray<T> {
    /**
     * Appends new elements to an array, and returns the new length of the array.
     *
     * @param items - new elements of the ObservableArray.
     *
     * @returns - number of the newly inserted items.
     */
    push: (...items: T[]) => number;
    /**
     * Removes the last element from an array and returns it.
     *
     * @returns - removed element or undefined if ObservableArray has no items.
     */
    pop: () => T | undefined;
    /**
     * Removes elements from an array and, if necessary, inserts new elements in their place, returning the deleted elements.
     *
     * @param start - Zero-based location in the array from which to start removing elements.
     *
     * @param deleteCount - Number of elements to remove.
     *
     * @param items - Elements to insert into the array in place of the deleted elements.
     *
     * @returns - deleted elements.
     */
    splice: (start: number, deleteCount: number, ...items: T[]) => T[];
    /**
     * Remove all items from the array that match the specified filter
     *
     * @param filter - Delegate which returns true for each item to remove. If undefined, all items in the array are removed.
     */
    removeAll(filter?: (item: T) => boolean): void;
    /**
     * Gets all the items in ObservableArray.
     */
    value: T[];
}
/**
 * WARNING: A copy of this interface also exists in WebPlatform\Web\extensions\vss-web\vss-platform\Context.ts
 * Please make sure that you keep both instances in sync.
 *
 * EventTypes:
 *  push - { addedItems }
 *  pop - { removedItems}
 *  splice - { addedItems, removedItems }
 */
export declare class ObservableArray<T> extends Observable<IObservableArrayEventArgs<T>> implements IObservableArray<T> {
    private internalItems;
    constructor(items?: T[]);
    push(...items: T[]): number;
    pop(): T | undefined;
    splice(start: number, deleteCount: number, ...itemsToAdd: T[]): T[];
    removeAll(filter?: (item: T) => boolean): void;
    indexOf(item: T, fromIndex?: number): number;
    value: T[];
    items: T[];
    readonly length: number;
}
