/**
 * function that does nothing and accepts any set of arguments.
 */
export function noop() { }
/**
 * Basic function for building a css classlist string from and array of classes, where
 * one of more of the arguments may be null or undefined.
 *
 * @param classes Array of strings the represents the css class list.
 *
 * @example css("base", "active", x === 42 && "optional") will return "base active optional" if x === 42 or "base active" otherwise
 */
export function css() {
    var classes = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        classes[_i] = arguments[_i];
    }
    return classes
        .filter(function (c) { return c; })
        .join(" ")
        .trim();
}
/**
 * Returns the set of parent elements with index 0 the root and the last
 * element is either the direct parent or itself based on includeSelf.
 *
 * @param element The element to get the parent element hierarchy from.
 * @param includeSelf Should the element supplied be included in the parent list.
 */
export function getParents(element, includeSelf) {
    var parentElements = [];
    if (includeSelf) {
        parentElements.push(element);
    }
    while (element.parentElement) {
        parentElements.splice(0, 0, element.parentElement);
        element = element.parentElement;
    }
    return parentElements;
}
/**
 * getRelationship returns the relationship of the two specified elements.
 *
 * @param element1
 * @param element2
 */
export function getRelationship(element1, element2) {
    // If the second element is a child of the first element, then element1 occurs before element2.
    if (element1.contains(element2)) {
        return 4 /* Parent */;
    }
    // If the first element is a child of the second element, then element1 occurs after element2.
    if (element2.contains(element1)) {
        return 3 /* Child */;
    }
    // Retrieve the parents of both the elements.
    var parents1 = getParents(element1, true);
    var parents2 = getParents(element2, true);
    for (var elementIndex = 0;; elementIndex++) {
        if (parents1[elementIndex] !== parents2[elementIndex]) {
            var siblings = parents1[elementIndex - 1].children;
            for (var siblingIndex = 0; siblingIndex < siblings.length; siblingIndex++) {
                if (siblings[siblingIndex] === parents1[elementIndex]) {
                    return 1 /* Before */;
                }
                if (siblings[siblingIndex] === parents2[elementIndex]) {
                    return 2 /* After */;
                }
            }
        }
    }
}
/**
 * preventDefault is used as a standard delegate to prevent the default behavior
 * for a given event.
 *
 * @param event Synthetic event that should have its default action prevented.
 */
export function preventDefault(event) {
    event.preventDefault();
}
/**
 * The FocusContainer class is used to track whether focus exists within
 * an element and its React children. If there are elements that are not in
 * the same element hierarchy, the caller should use a React portal to
 * ensure the proper bubbling of events.
 *
 * The owner should pass an onBlur method that is called when focus leaves
 * the current logical element hierarchy.
 */
var FocusContainer = /** @class */ (function () {
    function FocusContainer(onBlur) {
        var _this = this;
        this.focusTimeout = -1;
        this.focusCount = 0;
        /**
         * onBlur method that should be attached to the onBlur handler of the
         * continers root element.
         */
        this.onBlur = function (event) {
            _this.focusCount--;
            // Don't process blur's when the document doesnt have focus.
            if (document.hasFocus()) {
                if (_this.focusTimeout === -1) {
                    _this.focusTimeout = window.setTimeout(function () {
                        _this.focusTimeout = -1;
                        _this.onBlurHost(event);
                    }, 0);
                }
            }
        };
        /**
         * onFocus method that should be attached to the onFocus handler of the
         * continers root element.
         */
        this.onFocus = function (event) {
            _this.focusCount++;
            if (_this.focusTimeout !== -1) {
                window.clearTimeout(_this.focusTimeout);
                _this.focusTimeout = -1;
            }
        };
        this.onBlurHost = onBlur;
    }
    /**
     * close can be called to close the container and stop monitoring focus
     * changes. This should be called if components are in the process of
     * unmounting.
     */
    FocusContainer.prototype.close = function () {
        if (this.focusTimeout !== -1) {
            window.clearTimeout(this.focusTimeout);
            this.focusTimeout = -1;
        }
    };
    /**
     * isInside will tell you at this moment is the focus inside the component.
     * It may be in a transitional period between different children.
     */
    FocusContainer.prototype.isInside = function () {
        return this.focusCount > 0;
    };
    return FocusContainer;
}());
export { FocusContainer };
/**
 * Make sure the focus treatment is enabled and disabled based on
 * the state of mouse and keyboard usage.
 */
export function setFocusVisible(visible) {
    if (visible) {
        document.body.classList.add("bolt-focus-visible");
    }
    else {
        document.body.classList.remove("bolt-focus-visible");
    }
}
/* Setup the set of non-focus keys, when these are pressed it doesnt start showing focus treatment */
var nonFocusKeys = new Array(255);
nonFocusKeys[18 /* Alt */] = true;
nonFocusKeys[20 /* CapsLock */] = true;
nonFocusKeys[17 /* Ctrl */] = true;
nonFocusKeys[144 /* NumLock */] = true;
nonFocusKeys[19 /* Pause */] = true;
nonFocusKeys[145 /* ScrollLock */] = true;
nonFocusKeys[16 /* Shift */] = true;
nonFocusKeys[91 /* WindowsKey */] = true;
document.addEventListener("keydown", function (event) {
    if (!nonFocusKeys[event.which]) {
        setFocusVisible(true);
    }
}, true);
document.addEventListener("mousedown", function () { return setFocusVisible(false); }, true);
