// Copyright (C) Microsoft Corporation. All rights reserved.
define("VSSUI/Utilities/Observable/Observable", ["require", "exports"], function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    /**
     * An Observable implementation that will track a set of subscribers and supports
     * notifications when the underlying system changes.
     */
    var Observable = /** @class */ (function () {
        function Observable() {
            this.observers = {};
        }
        /**
         * notify is used to send the event to all subscribers that have signed up for this events
         * action. This means they have subscribed directly to this action, or to all actions.
         * If the caller requested the event be persisted the event will be fired in order to new
         * subscribers as well when they subscribe.
         *
         * @param event - The object that represents the event that has occurred.
         *
         * @param persistEvent - Optional value that determines if all future subscribers will
         *  recieve the event as well.
         */
        Observable.prototype.notify = function (value, action, persistEvent) {
            // NOTE: We need to make a copy of the observers since they may change during notification.
            if (this.observers[action]) {
                var observers = this.observers[action].slice();
                for (var observerIndex = 0; observerIndex < observers.length; observerIndex++) {
                    observers[observerIndex](value, action);
                }
            }
            if (this.observers[""]) {
                var observers = this.observers[""].slice();
                for (var observerIndex = 0; observerIndex < observers.length; observerIndex++) {
                    observers[observerIndex](value, action);
                }
            }
            // If the caller wants this event sent to all subscribers, event future ones, track it.
            if (persistEvent) {
                if (!this.events) {
                    this.events = [];
                }
                this.events.push({ action: action, value: value });
            }
        };
        Observable.prototype.subscribe = function (observer, action) {
            action = action || "";
            if (!this.observers[action]) {
                this.observers[action] = [];
            }
            this.observers[action].push(observer);
            // Fire the callback for any events that were persisted when they were sent.
            if (this.events) {
                for (var _i = 0, _a = this.events; _i < _a.length; _i++) {
                    var event_1 = _a[_i];
                    this.notify(event_1.value, event_1.action, false);
                }
            }
        };
        Observable.prototype.unsubscribe = function (observer, action) {
            action = action || "";
            if (this.observers[action]) {
                var observerIndex = this.observers[action].indexOf(observer);
                if (observerIndex >= 0) {
                    this.observers[action].splice(observerIndex, 1);
                }
            }
        };
        return Observable;
    }());
    exports.Observable = Observable;
    var ObservableValue = /** @class */ (function (_super) {
        __extends(ObservableValue, _super);
        function ObservableValue(value) {
            var _this = _super.call(this) || this;
            _this.v = value;
            return _this;
        }
        Object.defineProperty(ObservableValue.prototype, "value", {
            get: function () {
                return this.v;
            },
            set: function (value) {
                this.v = value;
                this.notify(value, "set");
            },
            enumerable: true,
            configurable: true
        });
        return ObservableValue;
    }(Observable));
    exports.ObservableValue = ObservableValue;
    var ObservableLike;
    (function (ObservableLike) {
        /**
         * Check whether the specified object is an observable or not.
         *
         * @param observableLike Object to perform observable check.
         */
        function isObservable(observableLike) {
            return observableLike && typeof observableLike.subscribe === "function";
        }
        ObservableLike.isObservable = isObservable;
        /**
         * Gets the value of the specified observable like. If not observable, returns the passed argument.
         *
         * @param observableLike Object to get the value.
         * @returns Observable value or the observable like itself.
         */
        function getValue(observableLike) {
            if (isObservable(observableLike)) {
                return observableLike.value;
            }
            return observableLike;
        }
        ObservableLike.getValue = getValue;
        /**
         * Subscribes to the specified object if it is an observable.
         *
         * @param observableLike Object to subscribe its value change if applicable.
         * @param observer Delegate to be executed when the underlying data changes.
         * @param action Optional argument that allows the consumer to supply a action
         *  with the delegate. If the action is supplied only those actions are delievered,
         *  while all actions are delivered is no action is supplied.
         */
        function subscribe(observableLike, observer, action) {
            if (isObservable(observableLike)) {
                observableLike.subscribe(observer, action);
            }
        }
        ObservableLike.subscribe = subscribe;
        /**
         * Unsubscribes from the specified object if it is an observable.
         *
         * @param observableLike Object to subscribe its value change if applicable.
         * @param observer Delegate to be executed when the underlying data changes.
         * @param action Optional argument that allows the consumer to supply a action
         *  with the delegate. If the action is supplied only those actions are delievered,
         *  while all actions are delivered is no action is supplied.
         */
        function unsubscribe(observableLike, observer, action) {
            if (isObservable(observableLike)) {
                observableLike.unsubscribe(observer, action);
            }
        }
        ObservableLike.unsubscribe = unsubscribe;
    })(ObservableLike = exports.ObservableLike || (exports.ObservableLike = {}));
    /**
     * WARNING: A copy of this interface also exists in WebPlatform\Web\extensions\vss-web\vss-platform\Context.ts
     * Please make sure that you keep both instances in sync.
     *
     * EventTypes:
     *  push - { addedItems }
     *  pop - { removedItems}
     *  splice - { addedItems, removedItems }
     */
    var ObservableArray = /** @class */ (function (_super) {
        __extends(ObservableArray, _super);
        function ObservableArray(items) {
            if (items === void 0) { items = []; }
            var _this = _super.call(this) || this;
            _this.internalItems = items || [];
            return _this;
        }
        ObservableArray.prototype.push = function () {
            var _a;
            var items = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                items[_i] = arguments[_i];
            }
            (_a = this.internalItems).push.apply(_a, items);
            this.notify({ addedItems: items }, "push");
            return items.length;
        };
        ObservableArray.prototype.pop = function () {
            var item = this.internalItems.pop();
            if (item !== undefined) {
                this.notify({ removedItems: [item] }, "pop");
            }
            return item;
        };
        ObservableArray.prototype.splice = function (start, deleteCount) {
            var _a;
            var itemsToAdd = [];
            for (var _i = 2; _i < arguments.length; _i++) {
                itemsToAdd[_i - 2] = arguments[_i];
            }
            var removedItems = (_a = this.internalItems).splice.apply(_a, [start, deleteCount].concat(itemsToAdd));
            this.notify({ addedItems: itemsToAdd, removedItems: removedItems }, "splice");
            return removedItems;
        };
        ObservableArray.prototype.indexOf = function (item, fromIndex) {
            return this.internalItems.indexOf(item, fromIndex);
        };
        Object.defineProperty(ObservableArray.prototype, "items", {
            get: function () {
                return this.internalItems.slice();
            },
            set: function (items) {
                this.splice.apply(this, [0, this.length].concat(items));
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(ObservableArray.prototype, "length", {
            get: function () {
                return this.internalItems.length;
            },
            enumerable: true,
            configurable: true
        });
        return ObservableArray;
    }(Observable));
    exports.ObservableArray = ObservableArray;
});
//# sourceMappingURL=Observable.js.map