import * as tslib_1 from "tslib";
import { Observable } from '../Core/Observable';
import { Filter, FILTER_CHANGE_EVENT } from './Filter';
import { ViewOptions } from './ViewOptions';
/**
 * Names of events fired by view state
 */
export var ViewStateEventNames;
(function (ViewStateEventNames) {
    /**
     * One or more members of the view state is changing
     */
    ViewStateEventNames.viewStateChanging = 'viewStateChanging';
    /**
     * One or more members of the view state was changed.
     */
    ViewStateEventNames.viewStateChanged = 'viewStateChanged';
})(ViewStateEventNames || (ViewStateEventNames = {}));
/**
 * Store for properties of a view
 */
var ViewState = /** @class */ (function () {
    function ViewState() {
        this.observable = new Observable();
    }
    ViewState.prototype.subscribe = function (observer, action) {
        this.observable.subscribe(observer, action);
    };
    ViewState.prototype.unsubscribe = function (observer, action) {
        this.observable.unsubscribe(observer, action);
    };
    /**
     * Notify listeners that a state value is changing. The sequence of events goes like this:
     *
     * 1. The 'viewStateChanging' event is fired
     * 2. The 'viewStateEventName' event is fired, letting consumers update state in reaction to the change.
     *    While these listeners are processing, individual state change events are suppresed.
     * 3. The observable event specified here is triggered
     * 4. The events from any state changed as part of the event listeners in step 2 are triggered.
     * 5. The 'viewStateChanged' event is fired
     *
     * @param eventValue The value of the state that was changed
     * @param viewStateEventName The name of the state-change event to fire
     * @param observable The observable that is firing the event
     * @param observableEventName Name of the specific event that should be triggered on the observable
     * @param observableNotify Method to directly notify the observable's consumers
     */
    ViewState.prototype.notify = function (eventValue, viewStateEventName, observable, observableEventName, observableNotify) {
        if (this.changingEvents) {
            // Already in the middle of a change event. Just queue the event.
            if (observable && observableNotify) {
                // If we have an existing event with this same observable/eventName, just update its value.
                var updatedExistingEvent = false;
                for (var _i = 0, _a = this.changingEvents; _i < _a.length; _i++) {
                    var existingEvent = _a[_i];
                    if (existingEvent.observable === observable && existingEvent.eventName === observableEventName) {
                        var viewStateObservable = observable;
                        if (viewStateObservable.mergeChangeEvents) {
                            existingEvent.eventValue = viewStateObservable.mergeChangeEvents(existingEvent.eventValue, eventValue);
                        }
                        else {
                            existingEvent.eventValue = eventValue;
                        }
                        updatedExistingEvent = true;
                        break;
                    }
                }
                // New observable/event. Add it to our queue.
                if (!updatedExistingEvent) {
                    this.changingEvents.push({
                        observable: observable,
                        eventName: observableEventName,
                        eventValue: eventValue,
                        notify: observableNotify
                    });
                }
            }
        }
        else {
            // Create a queue of callbacks. This queue will bock (but remember) events
            // from individual observables from firing until all the view-state-change
            // event listers have completed. Then the previously blocked events will be emitted.
            this.changingEvents = [];
            // Add this observable event first
            if (observable && observableNotify) {
                this.changingEvents.push({
                    observable: observable,
                    eventName: observableEventName,
                    eventValue: eventValue,
                    notify: observableNotify
                });
            }
            var viewStateChangeEvent = {
                eventName: viewStateEventName,
                eventValue: eventValue
            };
            // Fire an event to indicate that a view-state-change is starting
            this.observable.notify(viewStateChangeEvent, ViewStateEventNames.viewStateChanging);
            // Fire a change-start event
            this.observable.notify(eventValue, viewStateEventName);
            // Fire the individual callbacks that we previously suppressed
            for (var _b = 0, _c = this.changingEvents; _b < _c.length; _b++) {
                var changingEvent = _c[_b];
                changingEvent.notify(changingEvent.eventValue, changingEvent.eventName);
            }
            // Delete the changing events queue. The next change event will start this process.
            this.changingEvents = null;
            // Fire an event to indicate that a view-state-change is complete
            this.observable.notify(viewStateChangeEvent, ViewStateEventNames.viewStateChanged);
        }
    };
    return ViewState;
}());
export { ViewState };
/**
 * An observable value which is used in a view state
 */
var ViewStateObservable = /** @class */ (function (_super) {
    tslib_1.__extends(ViewStateObservable, _super);
    /**
     * Create a new view state observable
     *
     * @param notifyViewStateDelegate The view state's notify method
     * @param viewStateChangeEventName The name of the view state change event
     */
    function ViewStateObservable(notifyViewStateDelegate, viewStateChangeEventName, mergeEventsAsObjects, bypassCallback) {
        var _this = _super.call(this) || this;
        _this.notifyViewStateDelegate = notifyViewStateDelegate;
        _this.viewStateEventName = viewStateChangeEventName;
        _this.mergeEventsAsObjects = mergeEventsAsObjects;
        return _this;
    }
    ViewStateObservable.prototype.notify = function (value, action, persistEvent) {
        if (this.shouldBypassViewState(value, action, persistEvent)) {
            _super.prototype.notify.call(this, value, action, persistEvent);
        }
        else {
            this.notifyViewStateDelegate(value, this.viewStateEventName, this, action, _super.prototype.notify.bind(this));
        }
    };
    ViewStateObservable.prototype.shouldBypassViewState = function (value, action, persistEvent) {
        return false;
    };
    ViewStateObservable.prototype.mergeChangeEvents = function (firstEventValue, nextEventValue) {
        return nextEventValue;
    };
    return ViewStateObservable;
}(Observable));
export { ViewStateObservable };
/**
 * An observable value which is used in a view state
 */
var ViewStateObservableValue = /** @class */ (function (_super) {
    tslib_1.__extends(ViewStateObservableValue, _super);
    function ViewStateObservableValue(notifyViewStateDelegate, viewStateChangeEventName, value) {
        var _this = _super.call(this, notifyViewStateDelegate, viewStateChangeEventName) || this;
        _this.v = value;
        return _this;
    }
    Object.defineProperty(ViewStateObservableValue.prototype, "value", {
        get: function () {
            return this.v;
        },
        set: function (value) {
            if (value !== this.v) {
                this.v = value;
                this.notify(value, "set");
            }
        },
        enumerable: true,
        configurable: true
    });
    return ViewStateObservableValue;
}(ViewStateObservable));
export { ViewStateObservableValue };
/**
 * Filter used within a view state
 */
var ViewStateFilter = /** @class */ (function (_super) {
    tslib_1.__extends(ViewStateFilter, _super);
    function ViewStateFilter(notifyViewStateDelegate, viewStateChangeEventName, options) {
        return _super.call(this, options, new ViewStateFilterObservable(notifyViewStateDelegate, viewStateChangeEventName)) || this;
    }
    return ViewStateFilter;
}(Filter));
export { ViewStateFilter };
var ViewStateFilterObservable = /** @class */ (function (_super) {
    tslib_1.__extends(ViewStateFilterObservable, _super);
    function ViewStateFilterObservable() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ViewStateFilterObservable.prototype.shouldBypassViewState = function (value, action, persistEvent) {
        return action !== FILTER_CHANGE_EVENT;
    };
    ViewStateFilterObservable.prototype.mergeChangeEvents = function (firstEventValue, nextEventValue) {
        return tslib_1.__assign({}, firstEventValue, nextEventValue);
    };
    return ViewStateFilterObservable;
}(ViewStateObservable));
/**
 * ViewOptions used within a view state
 */
var ViewStateOptions = /** @class */ (function (_super) {
    tslib_1.__extends(ViewStateOptions, _super);
    function ViewStateOptions(notifyViewStateDelegate, viewStateChangeEventName, options) {
        return _super.call(this, options, new ViewStateOptionsObservable(notifyViewStateDelegate, viewStateChangeEventName)) || this;
    }
    return ViewStateOptions;
}(ViewOptions));
export { ViewStateOptions };
var ViewStateOptionsObservable = /** @class */ (function (_super) {
    tslib_1.__extends(ViewStateOptionsObservable, _super);
    function ViewStateOptionsObservable() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ViewStateOptionsObservable.prototype.mergeChangeEvents = function (firstEventValue, nextEventValue) {
        return tslib_1.__assign({}, firstEventValue, nextEventValue);
    };
    return ViewStateOptionsObservable;
}(ViewStateObservable));
